@@ -17,8 +17,12 @@ BROWSER_NAMES = {'ff': "_make_ff",
                  'gc': "_make_chrome",
                  'chrome': "_make_chrome",
                  'opera' : "_make_opera",
+                 'phantomjs' : "_make_phantomjs",
                  'htmlunit' : "_make_htmlunit",
-                 'htmlunitwithjs' : "_make_htmlunitwithjs"
+                 'htmlunitwithjs' : "_make_htmlunitwithjs",
+                 'android': "_make_android",
+                 'iphone': "_make_iphone",
+                 'safari': "_make_safari"
                 }
 
 class _BrowserManagementKeywords(KeywordGroup):
@@ -74,9 +78,13 @@ class _BrowserManagementKeywords(KeywordGroup):
         | gc               | Google Chrome |
         | chrome           | Google Chrome |
         | opera            | Opera         |
+        | phantomjs        | PhantomJS     |
         | htmlunit         | HTMLUnit      |
         | htmlunitwithjs   | HTMLUnit with Javascipt support |
-        
+        | android          | Android       |
+        | iphone           | Iphone        |
+        | safari           | Safari        |
+
 
         Note, that you will encounter strange behavior, if you open
         multiple Internet Explorer browser instances. That is also why
@@ -88,9 +96,10 @@ class _BrowserManagementKeywords(KeywordGroup):
         http://127.0.0.1/wd/hub.  If you specify a value for remote you can
         also specify 'desired_capabilities' which is a string in the form
         key1:val1,key2:val2 that will be used to specify desired_capabilities
-        to the remote server.  This is useful for doing things like specify a
+        to the remote server. This is useful for doing things like specify a
         proxy server for internet explorer or for specify browser and os if your
-        using saucelabs.com.
+        using saucelabs.com. 'desired_capabilities' can also be a dictonary
+        (created with 'Create Dictionary') to allow for more complex configurations.
 
         Optional 'ff_profile_dir' is the path to the firefox profile dir if you
         wish to overwrite the default.
@@ -107,6 +116,60 @@ class _BrowserManagementKeywords(KeywordGroup):
                     % browser.session_id)
         return self._cache.register(browser, alias)
 
+    def create_webdriver(self, driver_name, alias=None, kwargs={}, **init_kwargs):
+        """Creates an instance of a WebDriver.
+
+        Like `Open Browser`, but allows passing arguments to a WebDriver's
+        __init__. _Open Browser_ is preferred over _Create Webdriver_ when
+        feasible.
+
+        Returns the index of this browser instance which can be used later to
+        switch back to it. Index starts from 1 and is reset back to it when
+        `Close All Browsers` keyword is used. See `Switch Browser` for
+        example.
+
+        `driver_name` must be the exact name of a WebDriver in
+        _selenium.webdriver_ to use. WebDriver names include: Firefox, Chrome,
+        Ie, Opera, Safari, PhantomJS, and Remote.
+
+        Use keyword arguments to specify the arguments you want to pass to
+        the WebDriver's __init__. The values of the arguments are not
+        processed in any way before being passed on. For Robot Framework
+        < 2.8, which does not support keyword arguments, create a keyword
+        dictionary and pass it in as argument `kwargs`. See the
+        [http://selenium.googlecode.com/git/docs/api/py/api.html|Selenium API Documentation]
+        for information about argument names and appropriate argument values.
+
+        Examples:
+        | # use proxy for Firefox     |              |                                           |                         |
+        | ${proxy}=                   | Evaluate     | sys.modules['selenium.webdriver'].Proxy() | sys, selenium.webdriver |
+        | ${proxy.http_proxy}=        | Set Variable | localhost:8888                            |                         |
+        | Create Webdriver            | Firefox      | proxy=${proxy}                            |                         |
+        | # use a proxy for PhantomJS |              |                                           |                         |
+        | ${service args}=            | Create List  | --proxy=192.168.132.104:8888              |                         |
+        | Create Webdriver            | PhantomJS    | service_args=${service args}              |                         |
+        
+        Example for Robot Framework < 2.8:
+        | # debug IE driver |                   |                  |       |          |                       |
+        | ${kwargs}=        | Create Dictionary | log_level        | DEBUG | log_file | %{HOMEPATH}${/}ie.log |
+        | Create Webdriver  | Ie                | kwargs=${kwargs} |       |          |                       |
+        """
+        if not isinstance(kwargs, dict):
+            raise RuntimeError("kwargs must be a dictionary.")
+        for arg_name in kwargs:
+            if arg_name in init_kwargs:
+                raise RuntimeError("Got multiple values for argument '%s'." % arg_name)
+            init_kwargs[arg_name] = kwargs[arg_name]
+        driver_name = driver_name.strip()
+        try:
+            creation_func = getattr(webdriver, driver_name)
+        except AttributeError:
+            raise RuntimeError("'%s' is not a valid WebDriver name" % driver_name)
+        self._info("Creating an instance of the %s WebDriver" % driver_name)
+        driver = creation_func(**init_kwargs)
+        self._debug("Created %s WebDriver instance with session id %s" % (driver_name, driver.session_id))
+        return self._cache.register(driver, alias)
+
     def switch_browser(self, index_or_alias):
         """Switches between active browsers using index or alias.
 
@@ -169,6 +232,26 @@ class _BrowserManagementKeywords(KeywordGroup):
         """Maximizes current browser window."""
         self._current_browser().maximize_window()
 
+    def get_window_size(self):
+        """Returns current window size as `width` then `height`.
+
+        Example:
+        | ${width} | ${height}= | Get Window Size |
+        """
+        size = self._current_browser().get_window_size()
+        return size['width'], size['height']
+
+    def set_window_size(self, width, height):
+        """Sets the `width` and `height` of the current window to the specified values.
+
+        Example:
+        | Set Window Size | ${800} | ${600}       |
+        | ${width} | ${height}= | Get Window Size |
+        | Should Be Equal | ${width}  | ${800}    |
+        | Should Be Equal | ${height} | ${600}    |
+        """
+        return self._current_browser().set_window_size(width, height)
+
     def select_frame(self, locator):
         """Sets frame identified by `locator` as current frame.
 
@@ -342,7 +425,7 @@ class _BrowserManagementKeywords(KeywordGroup):
         """
         old_timeout = self.get_selenium_timeout()
         self._timeout_in_secs = robot.utils.timestr_to_secs(seconds)
-        for browser in self._cache.browsers:
+        for browser in self._cache.get_open_browsers():
             browser.set_script_timeout(self._timeout_in_secs)
         return old_timeout
 
@@ -388,22 +471,18 @@ class _BrowserManagementKeywords(KeywordGroup):
             raise RuntimeError('No browser is open')
         return self._cache.current
 
-    def _get_browser_token(self, browser_name):
-        return BROWSER_NAMES.get(browser_name.lower().replace(' ', ''), browser_name)
-
-
-    def _get_browser_creation_function(self,browser_name):
-        return BROWSER_NAMES.get(browser_name.lower().replace(' ', ''), browser_name)
-
-    def _make_browser(self , browser_name , desired_capabilities=None , profile_dir=None,
-                    remote=None):
+    def _get_browser_creation_function(self, browser_name):
+        func_name = BROWSER_NAMES.get(browser_name.lower().replace(' ', ''))
+        return getattr(self, func_name) if func_name else None
 
+    def _make_browser(self, browser_name, desired_capabilities=None,
+                      profile_dir=None, remote=None):
         creation_func = self._get_browser_creation_function(browser_name)
-        browser = getattr(self,creation_func)(remote , desired_capabilities , profile_dir)
 
-        if browser is None:
+        if not creation_func:
             raise ValueError(browser_name + " is not a supported browser.")
 
+        browser = creation_func(remote, desired_capabilities, profile_dir)
         browser.set_speed(self._speed_in_secs)
         browser.set_script_timeout(self._timeout_in_secs)
         browser.implicitly_wait(self._implicit_wait_in_secs)
@@ -434,6 +513,10 @@ class _BrowserManagementKeywords(KeywordGroup):
         return self._generic_make_browser(webdriver.Opera, 
                 webdriver.DesiredCapabilities.OPERA, remote, desired_capabilities)
 
+    def _make_phantomjs(self , remote , desired_capabilities , profile_dir):
+        return self._generic_make_browser(webdriver.PhantomJS, 
+                webdriver.DesiredCapabilities.PHANTOMJS, remote, desired_capabilities)
+
     def _make_htmlunit(self , remote , desired_capabilities , profile_dir):
         return self._generic_make_browser(webdriver.Remote, 
                 webdriver.DesiredCapabilities.HTMLUNIT, remote, desired_capabilities)
@@ -442,7 +525,18 @@ class _BrowserManagementKeywords(KeywordGroup):
         return self._generic_make_browser(webdriver.Remote, 
                 webdriver.DesiredCapabilities.HTMLUNITWITHJS, remote, desired_capabilities)
 
-    
+    def _make_android(self , remote , desired_capabilities , profile_dir):
+        return self._generic_make_browser(webdriver.Remote,
+                webdriver.DesiredCapabilities.ANDROID, remote, desired_capabilities)
+
+    def _make_iphone(self , remote , desired_capabilities , profile_dir):
+        return self._generic_make_browser(webdriver.Remote,
+                webdriver.DesiredCapabilities.IPHONE, remote, desired_capabilities)
+
+    def _make_safari(self , remote , desired_capabilities , profile_dir):
+        return self._generic_make_browser(webdriver.Safari,
+                webdriver.DesiredCapabilities.SAFARI, remote, desired_capabilities)
+
     def _generic_make_browser(self, webdriver_type , desired_cap_type, remote_url, desired_caps):
         '''most of the make browser functions just call this function which creates the 
         appropriate web-driver'''
@@ -451,20 +545,33 @@ class _BrowserManagementKeywords(KeywordGroup):
         else:
             browser = self._create_remote_web_driver(desired_cap_type,remote_url , desired_caps)
         return browser
-    
 
     def _create_remote_web_driver(self , capabilities_type , remote_url , desired_capabilities=None , profile=None):
+        '''parses the string based desired_capabilities if neccessary and
+        creates the associated remote web driver'''
+
+        desired_capabilities_object = capabilities_type.copy()
+
+        if type(desired_capabilities) in (str, unicode):
+            desired_capabilities = self._parse_capabilities_string(desired_capabilities)
+
+        desired_capabilities_object.update(desired_capabilities or {})
+
+        return webdriver.Remote(desired_capabilities=desired_capabilities_object,
+                command_executor=str(remote_url), browser_profile=profile)
+
+    def _parse_capabilities_string(self, capabilities_string):
         '''parses the string based desired_capabilities which should be in the form
-        key1:val1,key2:val2 and creates the associated remote web driver'''
-        desired_cap = self._create_desired_capabilities(capabilities_type , desired_capabilities)
-        return webdriver.Remote(desired_capabilities=desired_cap , command_executor=str(remote_url) ,                                       browser_profile=profile)
+        key1:val1,key2:val2
+        '''
+        desired_capabilities = {}
 
+        if not capabilities_string:
+            return desired_capabilities
+
+        for cap in capabilities_string.split(","):
+            (key, value) = cap.split(":", 1)
+            desired_capabilities[key.strip()] = value.strip()
 
-    def _create_desired_capabilities(self, capabilities_type, capabilities_string):
-        desired_capabilities = capabilities_type
-        if capabilities_string:
-            for cap in capabilities_string.split(","):
-                (key, value) = cap.split(":")
-                desired_capabilities[key.strip()] = value.strip()
         return desired_capabilities
     
